@@ -610,7 +610,8 @@ class AbstractSICConnector(object):
     def enable_service(self, name: str) -> None:
         """
         Enable the given service (for the previously selected devices)
-        :param name: people_detection, face_recognition, emotion_detection, corona_checker, intent_detection or sentiment_analysis
+        :param name: people_detection, face_recognition, emotion_detection, corona_checker, intent_detection,
+        sentiment_analysis
         :return:
         """
         pipe = self.redis.pipeline()
