@@ -1,3 +1,4 @@
+import json
 from enum import Enum
 from functools import partial
 from threading import Condition, Event, Thread
@@ -122,14 +123,16 @@ class BasicSICConnector(AbstractSICConnector):
 
         if sentiment:
             self.enable_service('sentiment_analysis')
+            self.final_sentiment = ''
         self.sentiment_enabled = sentiment
-        
+
         if dialogflow_language and dialogflow_key_file and dialogflow_agent_id:
             self.enable_service('intent_detection')
             sleep(1)  # give the service some time to load
             self.set_dialogflow_language(dialogflow_language)
             self.set_dialogflow_key(dialogflow_key_file)
             self.set_dialogflow_agent(dialogflow_agent_id)
+            self.final_text = ''
 
         if tts_voice:
             self.enable_service('text_to_speech')
@@ -221,7 +224,7 @@ class BasicSICConnector(AbstractSICConnector):
     ###########################
 
     def speech_recognition(self, context: str, max_duration: int, callback: callable = None,
-                           sentiment_callback: callable = None, sync: bool = True) -> None:
+                           with_sentiment=False, sync: bool = True) -> None:
         """
         Initiate a speech recognition attempt using Google's Dialogflow using a context.
         For more information on contexts see: https://cloud.google.com/dialogflow/docs/contexts-overview
@@ -232,6 +235,7 @@ class BasicSICConnector(AbstractSICConnector):
         :param context: Google's Dialogflow context label (str)
         :param max_duration: maximum time to listen in seconds (int)
         :param callback: callback function that will be called when a result (or fail) becomes available
+        :param with_sentiment:
         :param sync: lets main thread wait on this action
         """
         inner_callback = callback
@@ -239,12 +243,39 @@ class BasicSICConnector(AbstractSICConnector):
         if sync:
             inner_callback, lock = self.__build_sync_callback(inner_callback)
 
+        enhanced_callback, fail_callback, listen_lock = self.__build_speech_recording_callback(inner_callback,
+                                                                                               with_sentiment)
+        self.__register_action_listener('onAudioIntent', enhanced_callback)
+        self.__register_action_listener('IntentDetectionDone', fail_callback)
+
+        if self.sentiment_enabled and with_sentiment:
+            self.final_sentiment = ''
+            self.__register_action_listener('onTextSentiment', self.__sentiment_callback)
+
+        Thread(target=self.__recognizing, args=(context, listen_lock, max_duration)).start()
+
+        if lock:
+            lock.wait()
+
+    def __text_callback(self, text):
+        self.final_text = text
+
+    def __sentiment_callback(self, sentiment):
+        self.final_sentiment = sentiment
+
+    def speech_recognition_shortcut(self, context: str, shortcuts: dict, max_duration: int, callback: callable = None,
+                                    sync: bool = True) -> None:
+        inner_callback = callback
+        lock = None
+        if sync:
+            inner_callback, lock = self.__build_sync_callback(inner_callback)
+
         enhanced_callback, fail_callback, listen_lock = self.__build_speech_recording_callback(inner_callback)
         self.__register_action_listener('onAudioIntent', enhanced_callback)
         self.__register_action_listener('IntentDetectionDone', fail_callback)
 
-        if self.sentiment_enabled and sentiment_callback:
-            self.__register_action_listener('onTextSentiment', sentiment_callback)
+        shortcut_callback = self.__build_shortcut_callback(listen_lock, shortcuts, inner_callback)
+        self.__register_action_listener('onTextTranscript', shortcut_callback)
 
         Thread(target=self.__recognizing, args=(context, listen_lock, max_duration)).start()
 
@@ -278,6 +309,8 @@ class BasicSICConnector(AbstractSICConnector):
         lock.wait()
         self.__unregister_action_listener('onAudioIntent')
         self.__unregister_action_listener('IntentDetectionDone')
+        self.__unregister_action_listener('onTextTranscript')
+        self.__unregister_action_listener('onTextSentiment')
 
     def __recording(self, lock: Event, max_duration: int) -> None:
         self.stop_listening()
@@ -287,13 +320,14 @@ class BasicSICConnector(AbstractSICConnector):
         self.set_record_audio(False)
         self.__unregister_action_listener('onNewAudioFile')
 
-    @staticmethod
-    def __build_speech_recording_callback(embedded_callback: callable = None):
+    def __build_speech_recording_callback(self, embedded_callback: callable = None, with_sentiment: bool = False):
         lock = Event()
 
         def success_callback(*args):
             lock.set()
             if embedded_callback:
+                if with_sentiment:
+                    args += (self.final_sentiment,)
                 embedded_callback(*args)
 
         def fail_callback():
@@ -304,6 +338,17 @@ class BasicSICConnector(AbstractSICConnector):
 
         return success_callback, fail_callback, lock
 
+    @staticmethod
+    def __build_shortcut_callback(lock: Event, shortcuts: dict, embedded_callback: callable):
+        def shortcut_callback(transcript: str):
+            for answer in shortcuts.keys():
+                if any(variant in transcript for variant in shortcuts[answer]):
+                    if not lock.is_set():
+                        lock.set()
+                        if embedded_callback:
+                            embedded_callback(answer)
+        return shortcut_callback
+
     ###########################
     # Vision                  #
     ###########################
@@ -428,6 +473,28 @@ class BasicSICConnector(AbstractSICConnector):
         """
         self.__unregister_event_listener(event)
 
+    def wait(self, duration: int, callback: callable = None, sync: bool = True, load: bool = False):
+
+        inner_callback = callback
+        lock = None
+        if sync:
+            inner_callback, lock = self.__build_sync_callback(inner_callback)
+
+        self.__register_event_listener('onTimeout', inner_callback)
+        time_out_thread = Thread(target=self.__wait_action, args=(duration, ))
+
+        if load:
+            self.__loaded_actions.append(Action(time_out_thread.start, lock=lock))
+        else:
+            time_out_thread.start()
+            if lock:
+                lock.wait()
+
+    def __wait_action(self, duration: int):
+        sleep(duration)
+        self.__notify_event_listeners('onTimeout')
+        self.unsubscribe_event_listener('onTimeout')
+
     def __build_event_stopping_callback(self, event, original_callback):
         def callback(*args):
             if original_callback:
@@ -557,6 +624,12 @@ class BasicSICConnector(AbstractSICConnector):
         self.__process_action(super(BasicSICConnector, self).play_motion, motion,
                               callback=callback, event='PlayMotionDone', sync=sync, load=load)
 
+    def play_motion_file(self, motion_file: str, callback: callable = None,  sync: bool = True, load: bool = False):
+        with open(motion_file) as f:
+            motion = json.dumps(json.load(f))
+        self.__process_action(super(BasicSICConnector, self).play_motion, motion,
+                              callback=callback, event='PlayMotionDone', sync=sync, load=load)
+
     def start_record_motion(self, joint_chains: list, framerate: int = 5, callback: callable = None,  sync: bool = True, load: bool = False) -> None:
         self.__process_action(super(BasicSICConnector, self).start_record_motion, joint_chains, framerate,
                               callback=callback, event='RecordMotionStarted', sync=sync, load=load)
@@ -576,19 +649,22 @@ class BasicSICConnector(AbstractSICConnector):
         self.__action_listeners[event] = callback
 
     def __unregister_action_listener(self, event: str) -> None:
-        del self.__action_listeners[event]
+        if event in self.__action_listeners:
+            del self.__action_listeners[event]
 
     def __register_vision_listener(self, event: str, callback: callable) -> None:
         self.__vision_listeners[event] = callback
 
     def __unregister_vision_listener(self, event: str) -> None:
-        del self.__vision_listeners[event]
+        if event in self.__vision_listeners:
+            del self.__vision_listeners[event]
 
     def __register_event_listener(self, event: str, callback: callable):
         self.__event_listeners[event] = callback
 
     def __unregister_event_listener(self, event: str) -> None:
-        del self.__event_listeners[event]
+        if event in self.__event_listeners:
+            del self.__event_listeners[event]
 
     def __register_condition(self, condition: Condition) -> None:
         """
